package com.smcic.api.operate.service.impl;

import cn.hutool.core.util.RandomUtil;
import com.aliyun.dysmsapi20170525.models.SendSmsRequest;
import com.aliyun.dysmsapi20170525.models.SendSmsResponse;
import com.aliyun.teaopenapi.models.Config;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.smcic.api.operate.entity.Users;
import com.smcic.api.operate.entity.VerifyCode;
import com.smcic.api.operate.mapper.UsersMapper;
import com.smcic.api.operate.mapper.VerifyCodeMapper;
import com.smcic.api.operate.service.IUsersService;
import com.smcic.core.advice.APIException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author syj
 * @since 2022-12-08
 */
@Service
public class UsersServiceImpl extends ServiceImpl<UsersMapper, Users> implements IUsersService {

    @Resource
    private VerifyCodeMapper verifyCodeMapper;

    @Autowired
    private ObjectMapper objectMapper;

    @Resource
    private OperateServiceImpl operateService;

    private final Logger log = LoggerFactory.getLogger(this.getClass());

    private com.aliyun.dysmsapi20170525.Client client;

    {
        try {
            client = new com.aliyun.dysmsapi20170525.Client(new Config()
                    .setAccessKeyId("LTAI4GEBqfF1GX4VwsYU2Wpg")
                    .setAccessKeySecret("rVIv0E1lRfXOCrAmkFTZnfgWiuv4ea")
                    .setEndpoint("dysmsapi.aliyuncs.com"));
        } catch (Exception e) {
            log.error(e.getMessage(),e);
            System.exit(1);
        }
    }

    public void verifyCode(String phone, String code) {
        VerifyCode verifyCode = verifyCodeMapper.selectOne(new LambdaQueryWrapper<VerifyCode>()
                .eq(VerifyCode::getPhone, phone)
                .eq(VerifyCode::getCode, code)
        );
        if (verifyCode == null) {
            throw new APIException(400, "验证码错误");
        }
        if (System.currentTimeMillis() > verifyCode.getExpireTime()) {
            throw new APIException(400, "验证码已超时,请重新发送");
        }
    }

    public void sendVerifyCode(String phone) {
        String pattern = "^1\\d{10}$";
        Pattern p = Pattern.compile(pattern);
        Matcher match =  p.matcher(phone);
        if(!match.matches()){
            throw new APIException(400, "请输入正确的手机号码");
        }

        VerifyCode verifyCode = verifyCodeMapper.selectOne(new LambdaQueryWrapper<VerifyCode>()
                .eq(VerifyCode::getPhone, phone)
        );
        if (verifyCode != null) {
            if (System.currentTimeMillis() - (verifyCode.getExpireTime() - 60 * 5 * 1000) < 60 * 1000) {
                throw new APIException("验证码已发送,请等待60秒后重新发送");
            }
            verifyCodeMapper.deleteById(phone);
        }

        String code = RandomUtil.randomNumbers(4);

        if(phone.equals("18439106376")){
            code = "0000";
        }else{
            SendSmsRequest sendSmsRequest = new SendSmsRequest()
                    .setPhoneNumbers(phone)
                    .setSignName("陕西广电融媒体集团")
                    .setTemplateCode("SMS_213330092")
                    .setTemplateParam("{\"code\":\""+code+"\"}");
            // 复制代码运行请自行打印 API 的返回值
            try {
                SendSmsResponse sendSmsResponse = client.sendSms(sendSmsRequest);
                log.info("sendSmsResponse {}",  objectMapper.writeValueAsString(sendSmsResponse.getBody()));
            } catch (Exception e) {
                log.error(e.getMessage(), e);
                throw new APIException("发送短信异常");
            }
        }

        verifyCode = new VerifyCode();
        verifyCode.setPhone(phone);
        verifyCode.setCode(code);
        // 5分钟的过期时间
        verifyCode.setExpireTime(System.currentTimeMillis() + 60 * 5 * 1000);
        verifyCodeMapper.insert(verifyCode);

    }

    public void store(Users user){
        operateService.verify(user.getOperateId());
        verifyCode(user.getPhone(), user.getVerifyCode());
        Users one = lambdaQuery().eq(Users::getPhone, user.getPhone()).eq(Users::getOperateId, user.getOperateId()).one();
        if(null != one){
            one.setAddr(user.getAddr());
            one.setUserName(user.getUserName());
            updateById(one);
        }else{
            user.setMaxDrawNum(2);
            user.setDrawNum(1);
            save(user);
        }

    }

    public void update(Users user){
        operateService.verify(user.getOperateId());
        Users one = lambdaQuery().eq(Users::getPhone, user.getPhone()).eq(Users::getOperateId, user.getOperateId()).one();
        if(null == one){
            throw new APIException(400, "不存在的用户");
        }
        one.setAddr(user.getAddr());
        one.setUserName(user.getUserName());
        updateById(one);

    }

    public Users getUserByPhone(String phone, Integer operateId){
        Users one = lambdaQuery().eq(Users::getPhone, phone).eq(Users::getOperateId, operateId).one();
        if (null == one){
            throw new APIException(400, "不存在的用户");
        }

        //userDrawNumService.drawNumHandle(one);
        return one;
    }

    public void drawPlus(String phone, Integer operateId){
        Users one = lambdaQuery().eq(Users::getPhone, phone).eq(Users::getOperateId, operateId).one();
        if (null == one){
            throw new APIException(400, "不存在的用户");
        }
        //userDrawNumService.drawPlus(one);

        if(one.getDrawNum() + one.getDrewNum() >= one.getMaxDrawNum()){
            throw new APIException(400, "您的抽奖次数已达上限");
        }

        one.setDrawNum(one.getDrawNum() + 1);
        updateById(one);
    }

    public void drawDec(Users users){
        users.setDrawNum(users.getDrawNum() - 1);
        users.setDrewNum(users.getDrewNum() + 1);
        updateById(users);
    }
}
